<?php

namespace Drupal\amount_converter\Twig;

use Twig\Extension\AbstractExtension;
use Twig\TwigFilter;

/**
 * Class AmountConverterExtension.
 */
class AmountConverterExtension extends AbstractExtension {

  /**
   * {@inheritdoc}
   */
  public function getFilters() {
    return [
      new TwigFilter('number_to_chinese_uppercase', [$this, 'numberToChineseUppercase']),
    ];
  }

  /**
   * Converts a number to Chinese uppercase amount.
   *
   * @param float $number
   *   The number to convert.
   *
   * @return string
   *   The Chinese uppercase amount.
   */
  public function numberToChineseUppercase($number) {
    return $this->convertNumberToChinese($number);
  }

  /**
   * Converts a number to Chinese uppercase amount.
   *
   * @param float $num
   *   The number to convert.
   *
   * @return string
   *   The Chinese uppercase amount.
   */
  private function convertNumberToChinese($num) {
    // 确保输入是数字
    if (!is_numeric($num)) {
      return $num;
    }

    $num = round($num, 2); // 保留两位小数
    $cnums = ['零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖'];
    $grees = ['', '拾', '佰', '仟'];
    $int_units = ['', '万', '亿', '兆'];
    $dec_units = ['角', '分'];

    // 将数字转换为字符串并拆分整数和小数部分
    $num_str = (string)$num;
    $parts = explode('.', $num_str);
    $integer = $parts[0];
    $decimal = isset($parts[1]) ? $parts[1] : '';

    // 处理整数部分
    $integer_output = '';
    $len = strlen($integer);
    $zero = false;

    for ($i = 0; $i < $len; $i++) {
      $n = $integer[$i];
      $p = $len - $i - 1;
      $q = floor($p / 4); // 单位：万、亿、兆
      $r = $p % 4;        // 个、拾、佰、仟

      if ($n == '0') {
        $zero = true;
      } else {
        if ($zero) {
          $integer_output .= $cnums[0];
          $zero = false;
        }
        $integer_output .= $cnums[$n] . $grees[$r];
      }

      if ($r == 0 && $q > 0) {
        $integer_output .= $int_units[$q];
      }
    }

    if (empty($integer_output)) {
      $integer_output = $cnums[0];
    }
    $integer_output .= '元';

    // 处理小数部分
    $decimal_output = '';
    if ($decimal) {
      $decimal = str_pad($decimal, 2, '0', STR_PAD_RIGHT);
      for ($i = 0; $i < 2; $i++) {
        $n = $decimal[$i];
        if ($n != '0') {
          $decimal_output .= $cnums[$n] . $dec_units[$i];
        } else if ($i == 0 && $decimal[1] != '0') {
          // 处理角位为零且分位不为零的情况
          $decimal_output .= $cnums[0];
        }
      }
    }

    if (empty($decimal_output)) {
      $decimal_output .= '整';
    }

    return $integer_output . $decimal_output;
  }

}